using System;
using System.Data.Linq;
using System.Data.Linq.Mapping;

namespace DAODemo1.Processors
{
    public class BaseProcessor<T>
    {
        protected IDataContextFactory contextFactory;

        public BaseProcessor(IDataContextFactory contextFactory)
        {
            this.contextFactory = contextFactory;
        }

        protected string GetTableName()
        {
            object[] attributes = typeof (T).GetCustomAttributes(typeof (TableAttribute), false);
            foreach (var attribute in attributes)
            {
                return ((TableAttribute) attribute).Name;
            }
            return string.Empty;
        }

        protected void Process(Action<ITable> action)
        {
            using (DataContext ctx = contextFactory.CreateDataContext())
            {
                ITable table = ctx.GetTable(typeof(T));
                action(table);
                ctx.SubmitChanges();
            }
        }
    }
}