using System.Collections;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Reflection;

namespace DAODemo1.Processors
{
    public class FindProcessor<T> : BaseProcessor<T>, IProcessor
    {
        public FindProcessor(IDataContextFactory ctx) : base(ctx)
        {
        }

        public object Process(params object[] parameters)
        {
            using (DataContext ctx = contextFactory.CreateDataContext())
            {
                IEnumerable results = ctx.ExecuteQuery(typeof (T),
                                                       "SELECT * FROM " + GetTableName() + " WHERE " +
                                                       GetPrimaryCriteria(), parameters);
                foreach (var result in results)
                {
                    return result;
                }
                return null;
            }
        }

        private string GetPrimaryCriteria()
        {
            PropertyInfo[] properties = typeof (T).GetProperties();
            foreach (var property in properties)
            {
                object[] attributes = property.GetCustomAttributes(typeof (ColumnAttribute), false);
                foreach (var attribute in attributes)
                {
                    var colAtt = ((ColumnAttribute) attribute);
                    if (colAtt.IsPrimaryKey)
                    {
                        if (property.GetType() == typeof (string))
                        {
                            return property.Name + " = '{0}'";
                        }
                        return property.Name + " = {0}";
                    }
                }
            }
            return null;
        }
    }
}