using System;
using Castle.DynamicProxy;
using DAODemo1.DAOAttributes;

namespace DAODemo1.Processors
{
    public class ProcessorFactory
    {
        public static IProcessor GetProcessor(object att, IInvocation invocation, IDataContextFactory contextFactory)
        {
            Type processorType = null;
            object[] args = {contextFactory};
            if (att is FindAttribute)
            {
                processorType = typeof (FindProcessor<>);
            }
            if (att is DeleteAttribute)
            {
                processorType = typeof (DeleteProcessor<>);
            }
            if (att is CreateAttribute)
            {
                processorType = typeof (CreateProcessor<>);
            }
            if (att is QueryAttribute)
            {
                processorType = typeof (QueryProcessor<>);
                args = new object[] {contextFactory, ((QueryAttribute) att).Query};
            }
            Type type = processorType.MakeGenericType(GetDomainType(invocation));
            return (IProcessor) Activator.CreateInstance(type, args);
        }

        private static Type GetDomainType(IInvocation invocation)
        {
            foreach (var type in invocation.Proxy.GetType().GetInterfaces())
            {
                foreach (var attribute in type.GetCustomAttributes(false))
                {
                    if (attribute is EntityTypeAttribute)
                    {
                        return ((EntityTypeAttribute) attribute).DomainType;
                    }
                }
            }
            return null;
        }
    }
}