using System;
using System.Collections;
using System.Collections.Generic;
using System.Data.Linq;

namespace DAODemo1.Processors
{
    public class QueryProcessor<T> : BaseProcessor<T>, IProcessor
    {
        private readonly string queryString;

        public QueryProcessor(IDataContextFactory ctx, string queryString) :base(ctx)
        {
            this.queryString = queryString;
        }

        public object Process(params object[] parameters)
        {
            using (DataContext ctx = contextFactory.CreateDataContext())
            {
                IEnumerable results = ctx.ExecuteQuery(typeof (T),
                                                       "SELECT * FROM " + GetTableName() + " WHERE " +
                                                       string.Format(queryString, parameters));
                Type listType = typeof (List<>).MakeGenericType(typeof (T));
                object list = Activator.CreateInstance(listType);
                foreach (var result in results)
                {
                    list.GetType().GetMethod("Add").Invoke(list, new[] {result});
                }
                return list;
            }
        }
    }
}